# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marquez_client import MarquezWriteOnlyClient


# TODO: Method _path() is internal, but adding a test for now.
#  We need to add tests for write only client that mocks the backend
def test_path():
    path = MarquezWriteOnlyClient._path('/test/{0}', '1')

    assert path == '/api/v1/test/1'
